<?php

namespace Bydedi\Flux\GoogleShopping\Entity;

use Bydedi\Flux\GoogleShopping\Exception\EmptyParamException;
use Bydedi\Flux\GoogleShopping\Exception\LengthParamException;
use Bydedi\Flux\GoogleShopping\Exception\TypeParamException;
use Bydedi\Flux\GoogleShopping\Exception\UnauthorizedValueException;
use Exception;

/**
 * Class Product
 * @package Bydedi\Flux\GoogleShopping\Entity
 * Doc : https://support.google.com/merchants/answer/7052112
 */
class Product
{

    /**
     * Identifiant unique du produit
     * (format avec attribut : produit_id + ATT + produit_attribut_id)
     * Doc : https://support.google.com/merchants/answer/6324405
     *
     * @var string
     */
    protected $id;

    /**
     * Nom du produit
     * Doc : https://support.google.com/merchants/answer/6324415
     *
     * @var string
     */
    protected $title;

    /**
     * Description du produit
     * Limite : 5000 caractères
     * Doc : https://support.google.com/merchants/answer/6324468
     *
     * @var string
     */
    protected $description;

    /**
     * Identifiant du gorupe de produit
     * (produit_id uniquement dans le cas d'une déclinaison de produit par attribut)
     * Doc : https://support.google.com/merchants/answer/6324507
     *
     * @var string
     */
    protected $itemGroupId;

    /**
     * URL de l'image principale du produit
     * Doc : https://support.google.com/merchants/answer/6324350
     *
     * @var string
     */
    protected $imageLink;

    /**
     * URL d'une image supplémentaire du produit
     * Doc : https://support.google.com/merchants/answer/6324370
     *
     * @var string[]
     */
    protected $additionalImageLink;

    /**
     * Categorie de produit définie par Google
     * Doc : https://support.google.com/merchants/answer/6324436
     *
     * @var string
     */
    protected $googleProductCategory;

    /**
     * Disponibilité du produit
     * Valeurs autorisées : in stock / out of stock
     * Doc : https://support.google.com/merchants/answer/6324448
     *
     * @var string
     */
    protected $availability;

    /**
     * Prix du produit
     * Valeurs autorisées : price + currency (ex : 15.00 EUR)
     * Doc : https://support.google.com/merchants/answer/6324371
     *
     * @var string
     */
    protected $price;

    /**
     * Nom de la marque du produit
     * Doc : https://support.google.com/merchants/answer/6324351
     *
     * @var string
     */
    protected $brand;

    /**
     * Code GTIN du produit
     * Format autorisés : UPC / EAN / JAN / ISBN / ITF-14
     * Doc : https://support.google.com/merchants/answer/6324461
     *
     * @var string
     */
    protected $gtin;

    /**
     * Référence fabricant
     * (facultatif)
     * Doc : https://support.google.com/merchants/answer/6324482
     *
     * @var string
     */
    protected $mpn;

    /**
     * Etat du produit
     * Valeurs autorisées : new / refurbished / used
     * Doc : https://support.google.com/merchants/answer/6324469
     *
     * @var string
     */
    protected $condition;

    /**
     * Indique un produit à caractère sexuel
     * Valeurs autorisées : yes / no
     * Doc : https://support.google.com/merchants/answer/6324508
     *
     * @var string
     */
    protected $adult = "no";

    /**
     * Tranche d'âge cible du produit
     * Valeurs autorisées :
     *          newborn (0 - 3 mois)
     *          infant  (3 - 12 mois)
     *          toddler (1 - 5 ans)
     *          kids    (5 - 13 ans)
     *          adult
     * Doc : https://support.google.com/merchants/answer/6324463
     *
     * @var string
     */
    protected $ageGroup;

    /**
     * Categorie du produit propre au site
     * Doc : https://support.google.com/merchants/answer/6324406
     *
     * @var string
     */
    protected $productType;

    /**
     * Poids du colis
     * Format : entier + unité
     * (facultatif)
     * Doc : https://support.google.com/merchants/answer/6324503
     *
     * @var string
     */
    protected $shippingWeight;

    /**
     * Livraisons possibles du produit
     * Doc : https://support.google.com/merchants/answer/6324484
     *
     * @var Shipping[]
     */
    protected $shipping;

    /**
     * URL vers la page du produit
     * Doc : https://support.google.com/merchants/answer/6324416
     *
     * @var string
     */
    protected $link;


    /**
     * @return string
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @param string $id
     * @return Product $this
     * @throws Exception
     */
    public function setId($id)
    {
        if (empty($id)) {
            throw new EmptyParamException("id");
        }
        if (strlen($id) > 50) {
            throw new LengthParamException("id", strlen($id), 50);
        }

        $this->id = $id;

        return $this;
    }

    /**
     * @return string
     */
    public function getTitle()
    {
        return $this->title;
    }

    /**
     * @param string $title
     * @return Product $this
     * @throws Exception
     */
    public function setTitle($title)
    {
        if (empty($title)) {
            throw new EmptyParamException("title");
        }
        if (strlen($title) > 150) {
            throw new LengthParamException("id", strlen($title), 150);
        }

        $this->title = $title;

        return $this;
    }

    /**
     * @return string
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * @param string $description
     * @return Product $this
     * @throws Exception
     */
    public function setDescription($description)
    {
        if (empty($description)) {
            throw new EmptyParamException("description");
        }
        if (strlen($description) > 5000) {
            throw new LengthParamException("description", strlen($description), 5000);
        }

        $this->description = html_entity_decode($description, ENT_COMPAT, 'utf-8');

        return $this;
    }

    /**
     * @return string
     */
    public function getItemGroupId()
    {
        return $this->itemGroupId;
    }

    /**
     * @param string $itemGroupId
     * @return Product $this
     * @throws Exception
     */
    public function setItemGroupId($itemGroupId)
    {
        if (empty($itemGroupId)) {
            throw new EmptyParamException("item_group_id");
        }
        if (strlen($itemGroupId) > 50) {
            throw new LengthParamException("item_group_id", strlen($itemGroupId), 50);
        }

        $this->itemGroupId = $itemGroupId;

        return $this;
    }

    /**
     * @return string
     */
    public function getImageLink()
    {
        return $this->imageLink;
    }

    /**
     * @param string $imageLink
     * @return Product $this
     * @throws Exception
     */
    public function setImageLink($imageLink)
    {
        if (empty($imageLink)) {
            throw new EmptyParamException("image_link");
        }
        if (strlen($imageLink) > 2000) {
            throw new LengthParamException("image_link", strlen($imageLink), 2000);
        }
        // TODO verif extension fichiers (GIF, JPEG, PNG, BMP, TIFF only)

        $this->imageLink = $imageLink;

        return $this;
    }

    /**
     * @return \string[]
     */
    public function getAdditionalImageLink()
    {
        return $this->additionalImageLink;
    }

    /**
     * @param string[] $additionalImageLink
     * @return Product $this
     * @throws Exception
     */
    public function setAdditionalImageLink($additionalImageLink)
    {
        if (!is_array($additionalImageLink)) {
            throw new TypeParamException("additional_image_link", $additionalImageLink, "array");
        }

        foreach ($additionalImageLink as $link) {
            if (empty($link)) {
                throw new EmptyParamException("additional_image_link");
            }
            if (strlen($link) > 2000) {
                throw new LengthParamException("additional_image_link", strlen($link), 2000);
            }
            // TODO verif extension fichiers (GIF, JPEG, PNG, BMP, TIFF only)
        }

        $this->additionalImageLink = $additionalImageLink;

        return $this;
    }

    /**
     * @return string
     */
    public function getGoogleProductCategory()
    {
        return $this->googleProductCategory;
    }

    /**
     * @param string $googleProductCategory
     * @return Product
     */
    public function setGoogleProductCategory($googleProductCategory)
    {
        $this->googleProductCategory = $googleProductCategory;

        return $this;
    }

    /**
     * @return string
     */
    public function getAvailability()
    {
        return $this->availability;
    }

    /**
     * @param string $availability
     * @return Product $this
     * @throws Exception
     */
    public function setAvailability($availability)
    {
        $authorizedValues = array("in stock", "out of stock", "preorder");

        if (!in_array($availability, $authorizedValues)) {
            throw new UnauthorizedValueException("availability", $availability, $authorizedValues);
        }

        $this->availability = $availability;

        return $this;
    }

    /**
     * @return string
     */
    public function getPrice()
    {
        return $this->price;
    }

    /**
     * @param string $price
     * @return Product
     */
    public function setPrice($price)
    {
        $this->price = $price;

        return $this;
    }

    /**
     * @return string
     */
    public function getBrand()
    {
        return $this->brand;
    }

    /**
     * @param string $brand
     * @return Product $this
     * @throws Exception
     */
    public function setBrand($brand)
    {
        if (empty($brand)) {
            throw new EmptyParamException("brand");
        }
        if (strlen($brand) > 70) {
            throw new LengthParamException("brand", strlen($brand), 70);
        }

        $this->brand = $brand;

        return $this;
    }

    /**
     * @return string
     */
    public function getGtin()
    {
        return $this->gtin;
    }

    /**
     * @param string $gtin
     * @return Product $this
     * @throws Exception
     */
    public function setGtin($gtin)
    {
        $lengthGtin = strlen($gtin);
        if (empty($gtin)) {
            throw new EmptyParamException("gtin");
        }
        if ($lengthGtin != 8
            && $lengthGtin != 12
            && $lengthGtin != 13
            && $lengthGtin != 14
        ) {
            throw new LengthParamException("gtin", $lengthGtin, 0, array(8, 12, 13, 14));
        }

        $this->gtin = $gtin;

        return $this;
    }

    /**
     * @return string
     */
    public function getMpn()
    {
        return $this->mpn;
    }

    /**
     * @param string $mpn
     * @return Product
     * @throws Exception
     */
    public function setMpn($mpn)
    {
        if (empty($mpn)) {
            throw new EmptyParamException("mpn");
        }
        if (strlen($mpn) > 70) {
            throw new LengthParamException("mpn", strlen($mpn), 70);
        }

        $this->mpn = $mpn;

        return $this;
    }

    /**
     * @return string
     */
    public function getCondition()
    {
        return $this->condition;
    }

    /**
     * @param string $condition
     * @return Product
     * @throws Exception
     */
    public function setCondition($condition)
    {
        $authorizedValues = array("new", "refurbished", "used");

        if (!in_array($condition, $authorizedValues)) {
            throw new UnauthorizedValueException("condition", $condition, $authorizedValues);
        }

        $this->condition = $condition;

        return $this;
    }

    /**
     * @return string
     */
    public function getAdult()
    {
        return $this->adult;
    }

    /**
     * @param string $adult
     * @return Product
     * @throws Exception
     */
    public function setAdult($adult)
    {
        $authorizedValues = array("yes", "no");

        if (!in_array($adult, $authorizedValues)) {
            throw new UnauthorizedValueException("adult", $adult, $authorizedValues);
        }

        $this->adult = $adult;

        return $this;
    }

    /**
     * @return string
     */
    public function getAgeGroup()
    {
        return $this->ageGroup;
    }

    /**
     * @param string $ageGroup
     * @return Product
     * @throws Exception
     */
    public function setAgeGroup($ageGroup)
    {
        $authorizedvalues = array("newborn", "infant", "toddler", "kids", "adult");

        if (!in_array($ageGroup, $authorizedvalues)) {
            throw new UnauthorizedValueException("age_group", $ageGroup, $authorizedvalues);
        }
        $this->ageGroup = $ageGroup;

        return $this;
    }

    /**
     * @return string
     */
    public function getProductType()
    {
        return $this->productType;
    }

    /**
     * @param string $productType
     * @return Product
     * @throws Exception
     */
    public function setProductType($productType)
    {
        if (empty($productType)) {
            throw new EmptyParamException("product_type");
        }
        if (strlen($productType) > 750) {
            throw new LengthParamException("product_type", $productType, 750);
        }

        $this->productType = $productType;

        return $this;
    }

    /**
     * @return string
     */
    public function getShippingWeight()
    {
        return $this->shippingWeight;
    }

    /**
     * @param string $shippingWeight
     * @return Product
     */
    public function setShippingWeight($shippingWeight)
    {
        $this->shippingWeight = $shippingWeight;

        return $this;
    }


    /**
     * @return Shipping[]
     */
    public function getShipping()
    {
        return $this->shipping;
    }

    /**
     * @param Shipping[] $shipping
     * @return Product
     * @throws Exception
     */
    public function setShipping($shipping)
    {
        if (!is_array($shipping)) {
            throw new TypeParamException("shipping", $shipping, "array of object Shipping");
        }

        $this->shipping = $shipping;

        return $this;
    }

    /**
     * @return string
     */
    public function getLink()
    {
        return $this->link;
    }

    /**
     * @param string $link
     * @return Product
     * @throws Exception
     */
    public function setLink($link)
    {
        if (empty($link)) {
            throw new EmptyParamException("link");
        }
        if (strlen($link) > 2000) {
            throw new LengthParamException("link", strlen($link), 2000);
        }

        $this->link = $link;

        return $this;
    }
}